/* FxWrite.c
 *
 *  Written by: Steve Booth, Released to the public domain on 3/14/92...
 *
 *  This code lists the contents of a PARADOX table in Fixed Length Format.
 *  The table name should be included along with the program name.
 *
 *  Call:  FXWRITE table_name file_name {optional "8.2" number spec}
 *         the optional number specification allows you to adjust the
 *         formatting when you have numbers in the table.
 *
 *  FXWRITE also writes a crude text file (FIXFILE.DAT) describing the file
 *          layout.
 *
 *  This code must be compiled with the large model and uses Borland's 
 *       PARADOX engine... obviously not supplied.
*/

#include <conio.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pxengine.h>

#define NET_NAME_PATH           "c:\\apps\\paradox3"
#define NET_TYPE                NOTONNET
#define USER_NAME               "meuser"

#define SUCCESS		        0

#define MAX_FIELDS           255
#define MAX_BUFFER_SIZE      26
#define MAX_FILENAME_SIZE    40

// #define SEP  fprintf(ofileptr,",");
// #define EOR  fprintf(ofileptr,"#");

#define SEP  ;
#define EOR  ;

char table_name[100];
char file_name[100];

FILE *ofileptr = NULL;       // pointer to output file
FILE *fixptr = NULL;         // pointer to fixed output file
int  x;

//   Define Prototypes

void CloseTable(void);
void GetTableStructure(void);
void Intro(void);
int  Initialize(void);
int  main(int, char **);
void OpenTable(char *);
void gen_frmt(char [], int);
void PrintTable(void);
void PrintTableRecord(int);
void Terminate(void);

char         fieldNames[MAX_FIELDS][MAX_BUFFER_SIZE];
char         fieldTypes[MAX_FIELDS][MAX_BUFFER_SIZE];
char         fieldLengths[MAX_FIELDS][10];
char         fieldLength2[MAX_FIELDS][10];
RECORDNUMBER bytes;
int          nFields;
char         *frmt_str = "%8.2";

RECORDNUMBER nRecs;
RECORDHANDLE recHandle;
TABLEHANDLE  tableHandle;

int main(int argc, char **argv)
{
    char filename[MAX_FILENAME_SIZE + 1];
    int pxErr=0;

    system("cls");                          /* clear the screen          */
    if (Initialize() == SUCCESS)            /* initialize system         */
        {
        if (argc > 1)                       /* if table name included at */
        {
            OpenTable(argv[1]);             /* startup open the table    */
            strcpy(table_name, argv[1]);
        }
        else
        {
            Intro();                        /* display introduction text */
            printf("Enter table name: ");   /* prompt for table name     */
            scanf("%s",filename);
            OpenTable(filename);            /* attempt to open table     */
            strcpy(table_name, filename);
        }
        if (argc > 2)                       // if file name included at
        {
           ofileptr = fopen(argv[2],"wb");   // startup open the file
           strcpy(file_name, argv[2]);
        }
        else
            ofileptr = NULL;
        if (argc > 3)                       // if file name included at
        {
           strcpy(frmt_str, "%");
           strcat(frmt_str, argv[3]);
        }
        if ( !ofileptr )
        {
            printf("Enter file name: ");   /* prompt for file name     */
            scanf("%s",filename);
            strcpy(file_name, filename);
            ofileptr = fopen(filename,"wb");
        }

        if (!ofileptr) return(1);

        gotoxy(45,24);
        printf("Fixed Record File: FIXFILE.DAT");
        fixptr = fopen("FIXFILE.DAT","wb");
        GetTableStructure();                /* determines table structure*/
        PrintTable();                       /* prints records            */
        fprintf(fixptr, "\r\n\r\n\r\n");
		  fprintf(fixptr, "%ld Records, %3.3ld Characters/Record",
												  nRecs, bytes);
        fprintf(fixptr, "\r\n");
        CloseTable();                       /* closes table              */
        fclose(fixptr);
        Terminate();                        /* terminates the engine     */
        }
	 else
		  printf("%s\n", PXErrMsg(pxErr));    /* print error message if    */
														  /* the Engine could not be   */
														  /* initialized               */

    return(pxErr);                          /* exit and return value of  */
                                            /* Engine error code         */
}

int Initialize(void)
{
    return(PXNetInit(NET_NAME_PATH, NET_TYPE, USER_NAME));
}

void Intro()
{
    printf("\nThis is a program that reads from a Paradox table,\n"
           "and rewrites the data in Fixed Length Ascii File Format\n\n");
}


void OpenTable(char *tableName)
{
    if(PXTblOpen(tableName, &tableHandle, 0, 0))
    {
        printf("Cannot open table %s\n", tableName);
        exit(1);
    }
    PXRecBufOpen(tableHandle, &recHandle);      /* don't forget error chk */
}
void gen_frmt(char string[], int i)
{
char  format_str[10];
char *str_ptr;

	switch (string[0])
   {
		case 'A':
			  str_ptr = string + 1;
			  strcpy(format_str,"%-");
			  strcat(format_str, str_ptr);
              strcpy(fieldLength2[i],str_ptr);
           strcat(format_str,"s");
           break;

      case 'S':
			  strcpy(format_str,frmt_str);
              strcpy(fieldLength2[i],frmt_str+1);
              strcat(format_str,"d");
			  break;

      case 'N':
      case '$':
			  strcpy(format_str,frmt_str);
              strcpy(fieldLength2[i],frmt_str+1);
              strcat(format_str,"f");
			  break;

      case 'D':
			 strcpy(format_str,"");
              strcpy(fieldLength2[i],"10");
			 break;
	}
	strcpy(fieldLengths[i],format_str);
}
void GetTableStructure(void)
{
    int i;

    PXTblNRecs(tableHandle, &nRecs);
    PXRecNFlds(tableHandle, &nFields);
    for(i = 0; i < nFields; i++)
    {
        PXFldName(tableHandle, (FIELDHANDLE) i + 1,
                  MAX_BUFFER_SIZE, fieldNames[i]);
        PXFldType(tableHandle, (FIELDHANDLE) i + 1,
                  MAX_BUFFER_SIZE, fieldTypes[i]);
        gen_frmt(fieldTypes[i], i);
        fprintf(fixptr, "%c.....", fieldTypes[i][0]);
        fprintf(fixptr, "%s.....", fieldLength2[i]);
        fprintf(fixptr, "%s", fieldNames[i]);
        fprintf(fixptr, "\r\n");

    }
}

void PrintTable(void)
{
    int i;

    for(i = 1; i <= nRecs; i++)
    {
        PXRecGoto(tableHandle, i);
        PXRecGet(tableHandle, recHandle);
        PrintTableRecord(i);
    }
}

void PrintTableRecord(int rcdnum)
{
    int blank;
    int i;
    double dvalue;
    char avalue[BUFSIZ];
    int mo, da, yr, bout, bout2 = 0;
    long lvalue;
    short svalue;
    gotoxy(23,12);

    printf("Processing Record %d of %d\n", rcdnum, nRecs);

    for(i = 0; i < nFields; i++)
    {
       PXFldBlank(recHandle, i + 1, &blank);

          switch(fieldTypes[i][0])
          {

              case 'N':
              case '$':
						if ( !blank )
						{
							PXGetDoub(recHandle, i + 1, &dvalue);
						} else
							dvalue = 0;
							if ( strlen(fieldLengths[i]) != NULL )
							{
                            bout = fprintf(ofileptr, fieldLengths[i], dvalue);
							}
						break;

              case 'A':
						if ( !blank )
						{
							PXGetAlpha(recHandle, i + 1, BUFSIZ, avalue);
						} else
							strcpy(avalue, " ");

						if ( strlen(fieldLengths[i]) != NULL )
							{
                            bout = fprintf(ofileptr, fieldLengths[i], avalue);
							}
						break;

              case 'D':
						if ( !blank )
						{
							PXGetDate(recHandle, i + 1, &lvalue);
							PXDateDecode(lvalue, &mo, &da, &yr);
						}
						else
						{
							mo = 0;
							da = 0;
							yr = 0;
						}
                        bout = fprintf(ofileptr,"%2.2d/%2.2d/%4.4d", mo, da, yr);
                  break;

              case 'S':
						if ( !blank )
						{
							PXGetShort(recHandle, i + 1, &svalue);
						} else
							svalue = 0;
						if ( strlen(fieldLengths[i] )!= NULL )
							{
                            bout = fprintf(ofileptr, fieldLengths[i], svalue);
							}
						break;

              default:
                  printf("unknown\n");
                  break;
          }
			bout2 += bout;
		 SEP;
	 }
     if ( !bytes ) bytes = bout2;
     EOR;
}

void CloseTable(void)
{
    PXRecBufClose(recHandle);
    PXTblClose(tableHandle);
}

void Terminate(void)
{
    PXExit();
}
